module Term
  module ANSIColor
    class Attribute
      class IntenseColor8
        Attribute.set :intense_black,   90, direct: '#808080'
        Attribute.set :bright_black,    90, direct: '#808080'
        Attribute.set :intense_red,     91, direct: '#ff0000'
        Attribute.set :bright_red,      91, direct: '#ff0000'
        Attribute.set :intense_green,   92, direct: '#00ff00'
        Attribute.set :bright_green,    92, direct: '#00ff00'
        Attribute.set :intense_yellow,  93, direct: '#ffff00'
        Attribute.set :bright_yellow,   93, direct: '#ffff00'
        Attribute.set :intense_blue,    94, direct: '#0000ff'
        Attribute.set :bright_blue,     94, direct: '#0000ff'
        Attribute.set :intense_magenta, 95, direct: '#ff00ff'
        Attribute.set :bright_magenta,  95, direct: '#ff00ff'
        Attribute.set :intense_cyan,    96, direct: '#00ffff'
        Attribute.set :bright_cyan,     96, direct: '#00ffff'
        Attribute.set :intense_white,   97, direct: '#ffffff'
        Attribute.set :bright_white,    97, direct: '#ffffff'

        Attribute.set :on_intense_black,   100, direct: '#808080'
        Attribute.set :on_bright_black,    100, direct: '#808080'
        Attribute.set :on_intense_red,     101, direct: '#ff0000'
        Attribute.set :on_bright_red,      101, direct: '#ff0000'
        Attribute.set :on_intense_green,   102, direct: '#00ff00'
        Attribute.set :on_bright_green,    102, direct: '#00ff00'
        Attribute.set :on_intense_yellow,  103, direct: '#ffff00'
        Attribute.set :on_bright_yellow,   103, direct: '#ffff00'
        Attribute.set :on_intense_blue,    104, direct: '#0000ff'
        Attribute.set :on_bright_blue,     104, direct: '#0000ff'
        Attribute.set :on_intense_magenta, 105, direct: '#ff00ff'
        Attribute.set :on_bright_magenta,  105, direct: '#ff00ff'
        Attribute.set :on_intense_cyan,    106, direct: '#00ffff'
        Attribute.set :on_bright_cyan,     106, direct: '#00ffff'
        Attribute.set :on_intense_white,   107, direct: '#ffffff'
        Attribute.set :on_bright_white,    107, direct: '#ffffff'
      end
    end
  end
end
